#!/usr/bin/env python3
########################################################################
#
#  Copyright (c) 2021 by the contributors (see AUTHORS file)
#
#  This file is part of the CRISPRon
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  It is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this script, see file COPYING.
#  If not, see <http://www.gnu.org/licenses/>.
##########################################################################
import sys
import os
import shutil
import traceback
from collections import OrderedDict 
import numpy as np
from sklearn.model_selection import train_test_split
from scipy import stats
import tensorflow as tf
from scipy.spatial.distance import pdist,squareform,cdist
physical_devices = tf.config.list_physical_devices('GPU')
tf.config.experimental.set_memory_growth(physical_devices[0], True)
#gpu_device_name = tf.test.gpu_device_name()
#print(gpu_device_name)
#from tensorflow.python.client import device_lib

# 列出所有的本地机器设备
#import tensorflow as tf
#print("Num GPUs Available: ", len(tf.config.experimental.list_physical_devices('GPU')))


#%%
OPT = 'adam' #optimizer (eg. adam)
LEARN = 0.0001#learning rage eg.0.001
EPOCHS = 3000 # N. of epochs, eg 100
SEQ_C = '30mer_gRNA'# get seq from this column
VAL_C = 'Indel_freq_HEK293T' # get value to train from this column 
VAL_G = 'CRISPRoff_score' # get value to train from this column 


BATCH_SIZE= 100 # batch test 
SEED = 0# common random seed (0 equals randomly set)
TYPE = 'CG'# common random seed (0 equals randomly set)

print(
'OPT=%s' % OPT,
'LEARN=%f' % LEARN,
'EPOCHS=%i' % EPOCHS,
'SEQ_C=%s' % SEQ_C,
'VAL_C=%s' % VAL_C,
'VAL_G=%s' % VAL_G,

'BATCH_SIZE=%i' % BATCH_SIZE,
'SEED=%i' % SEED,
'TYPE=%s' % TYPE,
)


#length of input seq
eLENGTH=30
#depth of onehot encoding
eDEPTH=4


def read_seq_file(fn, seq_c0, val_c0, val_g0):
    d = OrderedDict()
    with open(fn, 'rt') as f:
        print(fn)
        head = f.readline().rstrip().split('\t')
        if seq_c0.isnumeric():
            seq_c = int(seq_c0) -1
        else:
            if seq_c0 in head:
                seq_c = head.index(seq_c0)
            else:
                raise Exception

        if val_c0.isnumeric():
            val_c = int(val_c0) -1
        else:
            if val_c0 in head:
                val_c = head.index(val_c0)
            else:
                raise Exception

        if val_g0.isnumeric():
            val_g = int(val_g0) -1
        else:
            if val_g0 in head:
                val_g = head.index(val_g0)
            else:
                raise Exception
        f.seek(0)

        for i,l in enumerate(f):
            v = l.rstrip().split('\t')
            s = v[seq_c]
            
            if not len(s) == eLENGTH:
                print(s)
                print('"%s" is not a string of length %d in line %d' % (s, eLENGTH, i+1))
                continue
            
            if s in d:
                print('"%s" is not unique in line %d', (s, i+1))
                continue

            try:
                e = float(v[val_c])
            except:
                print('no float value for "%s" in line %d', (s, i+1))
                e = 0.0
                continue
            try:
                g = float(v[val_g])
            except:
                print('no float value for "%s" in line %d', (s, i+1))
                g = 0.0
                continue

            d[s] = [g, e]
 
    return d

def onehot(x):
    z = list()
    for y in list(x):
        if y in "Aa":  z.append(0)
        elif y in "Cc": z.append(1)
        elif y in "Gg": z.append(2)
        elif y in "TtUu": z.append(3)
        else:

            raise Exception
    return z

def set_data(DX, s):
    for j,x in enumerate(onehot(s)):
        DX[j][x] = 1

def gived(data):
    dkeys = data.keys()
    DATA_Xori = np.zeros((len(dkeys),eLENGTH,eDEPTH), dtype=np.float32) # onehot
    DATA_X = np.zeros((len(dkeys),120), dtype=np.float32)

    for l, s in enumerate(dkeys):
        d = data[s]
        
        set_data(DATA_Xori[l], s)
        
    for ind,i in enumerate(DATA_Xori):
        DATA_X[ind] = i.flatten()

    d = pdist(DATA_X,'Hamming') 
    d = squareform(d)
    return d*120

def givedtraintestd(data):
    dkeys = data.keys()
    DATA_Xori = np.zeros((len(dkeys),20,eDEPTH), dtype=np.float32) # onehot
    DATA_X = np.zeros((len(dkeys),80), dtype=np.float32)

    for l, s in enumerate(dkeys):
        d = data[s]

        set_data(DATA_Xori[l], s[4:24])

    for ind,i in enumerate(DATA_Xori):
        DATA_X[ind] = i.flatten()
    pool = [i for i in range(23902)]
    test = [i for i in range(23313,23839)]
    for i in test:
        pool.remove(i)
    d = cdist(DATA_X[test],DATA_X[pool],'Hamming') 

    return d*80


tinput = list()
vinput = list()
ttinput = list()
print('reading training data')

#%%



#%%
##单独的cell
import heapq


pool = [i for i in range(23902)]
test = [i for i in range(23313,23839)]
for i in test:
    pool.remove(i)


d2 = read_seq_file('trainingwithoff.TSV', SEQ_C, VAL_C, VAL_G)
d2 = givedtraintestd(d2)
print(d2.shape)
print(d2)
len(d2)
#%%
ns = []
for i,item in enumerate(test):

    if  (min(d2[i]) <= 6) :
        print(item,min(d2[i]))
        ns.append(item)

print(ns)

for i in ns:
    test.remove(i)
#%%
print(len(test))
with open('KIMtest.txt', 'w') as f:
    for item in test:
        f.write("%s\n" % item)